package ldutil

import (
	"bytes"

	dagpb "gitlab.dms3.io/ld/go-codec-dagpb"
	ld "gitlab.dms3.io/ld/go-ld-prime"
	"gitlab.dms3.io/ld/go-ld-prime/codec/dagcbor"
	_ "gitlab.dms3.io/ld/go-ld-prime/codec/raw"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"
	basicnode "gitlab.dms3.io/ld/go-ld-prime/node/basic"
	"gitlab.dms3.io/ld/go-ld-prime/traversal/selector"
	ldselector "gitlab.dms3.io/ld/go-ld-prime/traversal/selector"
)

var defaultChooser = func(lnk ld.Link, lctx ld.LinkContext) (ld.NodePrototype, error) {
	// We can decode all nodes into basicnode's Any, except for
	// dagpb nodes, which must explicitly use the PBNode prototype.
	if lnk, ok := lnk.(cidlink.Link); ok && lnk.Cid.Prefix().Codec == 0x70 {
		return dagpb.Type.PBNode, nil
	}
	return basicnode.Prototype.Any, nil
}

func EncodeNode(node ld.Node) ([]byte, error) {
	var buffer bytes.Buffer
	err := dagcbor.Encode(node, &buffer)
	if err != nil {
		return nil, err
	}
	return buffer.Bytes(), nil
}

func DecodeNode(encoded []byte) (ld.Node, error) {
	nb := basicnode.Prototype.Any.NewBuilder()
	if err := dagcbor.Decode(nb, bytes.NewReader(encoded)); err != nil {
		return nil, err
	}
	return nb.Build(), nil
}

func ParseSelector(selector ld.Node) (selector.Selector, error) {
	return ldselector.ParseSelector(selector)
}
