package metadata

import (
	"bytes"
	"math/rand"
	"testing"

	"github.com/stretchr/testify/require"
	"gitlab.dms3.io/ld/go-ld-prime/codec/dagcbor"
	"gitlab.dms3.io/ld/go-ld-prime/fluent"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"
	basicnode "gitlab.dms3.io/ld/go-ld-prime/node/basic"

	"gitlab.dms3.io/dms3/go-graphsync/testutil"
)

func TestDecodeEncodeMetadata(t *testing.T) {
	cids := testutil.GenerateCids(10)
	initialMetadata := make(Metadata, 0, 10)
	nd := fluent.MustBuildList(basicnode.Prototype.List, 10, func(fla fluent.ListAssembler) {
		for _, k := range cids {
			blockPresent := rand.Int31()%2 == 0
			initialMetadata = append(initialMetadata, Item{k, blockPresent})
			fla.AssembleValue().CreateMap(2, func(fma fluent.MapAssembler) {
				fma.AssembleEntry("link").AssignLink(cidlink.Link{Cid: k})
				fma.AssembleEntry("blockPresent").AssignBool(blockPresent)
			})
		}
	})

	// verify metadata matches
	encoded, err := EncodeMetadata(initialMetadata)
	require.NoError(t, err, "encode errored")

	decodedMetadata, err := DecodeMetadata(encoded)
	require.NoError(t, err, "decode errored")
	require.Equal(t, initialMetadata, decodedMetadata, "metadata changed during encoding and decoding")

	// verify metadata is equivalent of LD node encoding
	encodedNode := new(bytes.Buffer)
	err = dagcbor.Encode(nd, encodedNode)
	require.NoError(t, err)
	decodedMetadataFromNode, err := DecodeMetadata(encodedNode.Bytes())
	require.NoError(t, err)
	require.Equal(t, decodedMetadata, decodedMetadataFromNode, "metadata not equal to LD encoding")

	nb := basicnode.Prototype.List.NewBuilder()
	err = dagcbor.Decode(nb, encodedNode)
	require.NoError(t, err)
	decodedNode := nb.Build()
	require.Equal(t, nd, decodedNode)
	nb = basicnode.Prototype.List.NewBuilder()
	err = dagcbor.Decode(nb, bytes.NewReader(encoded))
	require.NoError(t, err)
	decodedNodeFromMetadata := nb.Build()
	require.Equal(t, decodedNode, decodedNodeFromMetadata, "deserialzed metadata does not match deserialized node")
}
