package storeutil

import (
	"io/ioutil"
	"testing"

	"github.com/stretchr/testify/require"
	blocks "gitlab.dms3.io/dms3/go-block-format"
	"gitlab.dms3.io/dms3/go-datastore"
	dss "gitlab.dms3.io/dms3/go-datastore/sync"
	bstore "gitlab.dms3.io/dms3/go-dms3-blockstore"
	ld "gitlab.dms3.io/ld/go-ld-prime"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"

	"gitlab.dms3.io/dms3/go-graphsync/testutil"
)

func TestLinkSystem(t *testing.T) {
	store := bstore.NewBlockstore(dss.MutexWrap(datastore.NewMapDatastore()))
	blk := testutil.GenerateBlocksOfSize(1, 1000)[0]
	persistence := LinkSystemForBlockstore(store)
	buffer, commit, err := persistence.StorageWriteOpener(ld.LinkContext{})
	require.NoError(t, err, "Unable to setup buffer")
	_, err = buffer.Write(blk.RawData())
	require.NoError(t, err, "Unable to write data to buffer")
	err = commit(cidlink.Link{Cid: blk.Cid()})
	require.NoError(t, err, "Unable to put block to store")
	data, err := persistence.StorageReadOpener(ld.LinkContext{}, cidlink.Link{Cid: blk.Cid()})
	require.NoError(t, err, "Unable to load block with loader")
	bytes, err := ioutil.ReadAll(data)
	require.NoError(t, err, "Unable to read bytes from reader returned by loader")
	_, err = blocks.NewBlockWithCid(bytes, blk.Cid())
	require.NoError(t, err, "Did not return correct block with loader")
}
