package testutil

import (
	"bytes"
	"fmt"
	"io"
	"sync"

	ld "gitlab.dms3.io/ld/go-ld-prime"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"
)

// NewTestStore provides a loader and storer for the given in memory link -> byte data map
func NewTestStore(blocksWritten map[ld.Link][]byte) ld.LinkSystem {
	var storeLk sync.RWMutex
	lsys := cidlink.DefaultLinkSystem()
	lsys.TrustedStorage = true
	lsys.StorageWriteOpener = func(lnkCtx ld.LinkContext) (io.Writer, ld.BlockWriteCommitter, error) {
		var buffer bytes.Buffer
		committer := func(lnk ld.Link) error {
			storeLk.Lock()
			blocksWritten[lnk] = buffer.Bytes()
			storeLk.Unlock()
			return nil
		}
		return &buffer, committer, nil
	}
	lsys.StorageReadOpener = func(lnkCtx ld.LinkContext, lnk ld.Link) (io.Reader, error) {
		storeLk.RLock()
		data, ok := blocksWritten[lnk]
		storeLk.RUnlock()
		if ok {
			return bytes.NewReader(data), nil
		}
		return nil, fmt.Errorf("unable to load block")
	}

	return lsys
}
